using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TheLandBeforeDataBinding
{
    public partial class ComplexBindingListBoxForm : Form
    {
        List<RaceCarDriverNotify> drivers = new List<RaceCarDriverNotify>();

        BindingManagerBase BindingManager
        {
            get { return this.BindingContext[this.drivers]; }
        }

        public ComplexBindingListBoxForm()
        {
            InitializeComponent();
            
            drivers.Add(new RaceCarDriverNotify("M Schumacher", 500));
            drivers.Add(new RaceCarDriverNotify("A Senna", 1000));
            drivers.Add(new RaceCarDriverNotify("A Prost", 400));

            this.textBoxName.DataBindings.Add(
                "Text", drivers, "Name");
            this.textBoxWins.DataBindings.Add(
                            "Text", drivers, "Wins");

            this.listBoxDrivers.DataSource = drivers;
            radioButtonDefault.Checked = true;

            RefreshItems();
        }

        void RefreshItems()
        {
            int count = this.BindingManager.Count;
            int pos = this.BindingManager.Position + 1;

            this.labelCount.Text = count.ToString();
            this.labelPosition.Text = pos.ToString();

            this.buttonFirst.Enabled = (pos > 1);
            this.buttonPrev.Enabled = (pos > 1);
            this.buttonNext.Enabled = (pos < count);
            this.buttonLast.Enabled = (pos < count);
        }

        private void buttonFirst_Click(object sender, EventArgs e)
        {
            this.BindingManager.Position = 0;
            RefreshItems();
        }

        private void buttonPrev_Click(object sender, EventArgs e)
        {
            --this.BindingManager.Position;
            RefreshItems();
        }

        private void buttonNext_Click(object sender, EventArgs e)
        {
            ++this.BindingManager.Position;
            RefreshItems();
        }

        private void buttonLast_Click(object sender, EventArgs e)
        {
            this.BindingManager.Position = this.BindingManager.Count - 1;
            RefreshItems();
        }

        private void radioButtonDefault_CheckedChanged(object sender, EventArgs e)
        {
            listBoxDrivers.DisplayMember = "";
        }

        private void radioButtonName_CheckedChanged(object sender, EventArgs e)
        {
            listBoxDrivers.DisplayMember = "Name";
        }

        private void radioButtonWins_CheckedChanged(object sender, EventArgs e)
        {
            listBoxDrivers.DisplayMember = "Wins";
        }
    }
}