using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TheLandBeforeDataBinding
{
    public partial class TheLandBeforeDataBindingForm : Form
    {


        public TheLandBeforeDataBindingForm()
        {
            InitializeComponent();

            InitializeFirstExample();
            InitializeNotifyExample();
            InitializeSimpleBinding();
            InitializeBindingProperty();
            InitializeListDrivers();
            InitializeListData();
            InitializeComplexList();
            InitializeDataGrid();
            InitializeGridBindingList();
            InitializeGridBindingDefault();

        }

        #region First Example
        RaceCarDriver driver = new RaceCarDriver("M Schumacher", 500);

        void InitializeFirstExample()
        {
            this.nameTextBox.Text = this.driver.Name;
            this.winsTextBox.Text = this.driver.Wins.ToString();

            this.nameTextBox.TextChanged += nameTextBox_TextChanged;
            this.winsTextBox.TextChanged += winsTextBox_TextChanged;
        }

        void nameTextBox_TextChanged(object sender, EventArgs e)
        {
            this.driver.Name = this.nameTextBox.Text;
        }

        void winsTextBox_TextChanged(object sender, EventArgs e)
        {
            int wins = 0;
            int.TryParse(this.winsTextBox.Text, out wins);
            this.driver.Wins = wins;
        }

        private void buttonFirstExampleAddWin_Click(object sender, EventArgs e)
        {
            this.driver.Wins++;
            this.winsTextBox.Text = this.driver.Wins.ToString();
        }
        #endregion

        #region Notify Example
        RaceCarDriverNotify driverNotify = new RaceCarDriverNotify(
            "M Schumacher", 500);

        private void InitializeNotifyExample()
        {
            this.nameNotifyTextBox.Text = driverNotify.Name;
            this.winsNotifyTextBox.Text = driverNotify.Wins.ToString();

            this.nameNotifyTextBox.TextChanged += nameNotifyTextBox_TextChanged;
            this.winsNotifyTextBox.TextChanged += winsNotifyTextBox_TextChanged;

            this.driverNotify.PropertyChanged += driverNotifyExample_PropertyChanged;
        }


        void nameNotifyTextBox_TextChanged(object sender, EventArgs e)
        {
            this.driver.Name = this.nameTextBox.Text;
        }

        void winsNotifyTextBox_TextChanged(object sender, EventArgs e)
        {
            int wins = 0;
            int.TryParse(this.winsTextBox.Text, out wins);
            this.driver.Wins = wins;
        }

        void driverNotifyExample_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "Name":
                    this.nameNotifyTextBox.Text = this.driverNotify.Name;
                    break;
                case "Wins":
                    this.winsNotifyTextBox.Text = this.driverNotify.Wins.ToString();
                    break;
            }
        }

        private void buttonNotifyAddWin_Click(object sender, EventArgs e)
        {
            driverNotify.Wins++;
        }
        #endregion

        #region Simple Binding
        private void InitializeSimpleBinding()
        {
            Binding nameBinding =
                new Binding("Text", driverNotify, "Name", true);
            Binding winsBinding =
                new Binding("Text", driverNotify, "Wins", true);
            nameSimpleTextBox.DataBindings.Add(nameBinding);
            winsSimpleTextBox.DataBindings.Add(winsBinding);
        }

        private void buttonSimpleAddWins_Click(object sender, EventArgs e)
        {
            driverNotify.Wins++;
        }


        #endregion

        #region Control DataBindingProperty
        private void InitializeBindingProperty()
        {
            namePropertyTextBox.DataBindings.Add(
                "Text", this.driverNotify, "Name");
            winsPropertyTextBox.DataBindings.Add(
                    "Text", this.driverNotify, "Wins");
        }

        private void buttonPropertyAddWins_Click(object sender, EventArgs e)
        {
            driverNotify.Wins++;
        }
        #endregion

        #region List Data Source

        List<RaceCarDriverNotify> listDrivers = 
            new List<RaceCarDriverNotify>();

        private void InitializeListDrivers()
        {
            listDrivers.Add(new RaceCarDriverNotify("M. Schumacher", 500));
            listDrivers.Add(new RaceCarDriverNotify("A. Senna", 1000));
            listDrivers.Add(new RaceCarDriverNotify("A. Prost", 400));
        }

        private void InitializeListData()
        {
            nameListTextBox.DataBindings.Add("Text", listDrivers, "Name");
            winsListTextBox.DataBindings.Add("Text", listDrivers, "Wins");

            buttonListAddWins.Click += buttonListAddWins_Click;
        }

        void buttonListAddWins_Click(object sender, EventArgs e)
        {
            Current.Wins++;
        }

        BindingManagerBase BindingManager
        {
            get { return this.BindingContext[this.listDrivers]; }
        }

        RaceCarDriverNotify Current
        {
            get { return (RaceCarDriverNotify)BindingManager.Current; }
        }

        private void buttonFirst_Click(object sender, EventArgs e)
        {
            BindingManager.Position = 0;
        }

        private void buttonLast_Click(object sender, EventArgs e)
        {
            BindingManager.Position = BindingManager.Count - 1;
        }

        private void buttonPrevious_Click(object sender, EventArgs e)
        {
            BindingManager.Position--;
        }

        private void buttonNext_Click(object sender, EventArgs e)
        {
            BindingManager.Position++;
        }
        #endregion

        #region Complex Binding to List Data Source
        private void InitializeComplexList()
        {
            listBoxComplex.DataSource = listDrivers;
            listBoxComplex.DisplayMember = "Name";

            nameComplexTextBox.DataBindings.Add("Text", listDrivers, "Name");
            winsComplexTextBox.DataBindings.Add("Text", listDrivers, "Wins");
        }
        #endregion

        #region DataGrid
        private void InitializeDataGrid()
        {
            dataGridViewDrivers.DataSource = listDrivers;

            nameGridTextBox.DataBindings.Add("Text", listDrivers, "Name");
            winsGridTextBox.DataBindings.Add("Text", listDrivers, "Wins");
        }
        #endregion

        #region IBindingList
        BindingList<RaceCarDriverNotify> listBindingDrivers =
            new BindingList<RaceCarDriverNotify>();
        private void InitializeIBindingList()
        {
            listBindingDrivers.Add(
                new RaceCarDriverNotify("M. Schumacher", 500));
            listBindingDrivers.Add(
                new RaceCarDriverNotify("A. Senna", 1000));
            listBindingDrivers.Add(
                new RaceCarDriverNotify("A. Prost", 400));
                
        }

        BindingManagerBase BindingManagerBindingList
        {
            get { return this.BindingContext[this.listBindingDrivers]; }
        }

        RaceCarDriverNotify CurrentBindingList
        {
            get { return (RaceCarDriverNotify)BindingManagerBindingList.Current; }
        }
        #endregion       

        #region DataGridView with BindingList

        private void InitializeGridBindingList()
        {
            InitializeIBindingList();

            dataGridViewBinding.DataSource = listBindingDrivers;

            nameBindingTextBox.DataBindings.Add("Text", listBindingDrivers, "Name");
            winsBindingTextBox.DataBindings.Add("Text", listBindingDrivers, "Wins");
        }

        private void buttonAddWinsGrid_Click(object sender, EventArgs e)
        {
            CurrentBindingList.Wins++;
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            try
            {
                listBindingDrivers.AddNew();
            }
            catch (MissingMethodException mme)
            {
                MessageBox.Show("Error: no default constructor\n\r" + mme);
            }
            catch (InvalidOperationException ioe)
            {
                MessageBox.Show("Error: no default constructor\n\r" + ioe);
            }
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            listBindingDrivers.RemoveAt(BindingManagerBindingList.Position);
        }

        private void buttonFirstBindingList_Click(object sender, EventArgs e)
        {
            BindingManagerBindingList.Position = 0;
        }

        private void buttonPreviousBindingList_Click(object sender, EventArgs e)
        {
            BindingManagerBindingList.Position--;
        }

        private void buttonNextBindingList_Click(object sender, EventArgs e)
        {
            BindingManagerBindingList.Position++;
        }

        private void buttonLastBindingList_Click(object sender, EventArgs e)
        {
            BindingManagerBindingList.Position =
                BindingManagerBindingList.Count - 1;
        }
        #endregion

        #region IBindingList Default Constructor
        BindingList<RaceCarDriverDefault> listBindingDefault =
            new BindingList<RaceCarDriverDefault>();
        private void InitializeBindingDefault()
        {
            listBindingDefault.Add(
                new RaceCarDriverDefault("M. Schumacher", 500));
            listBindingDefault.Add(
                new RaceCarDriverDefault("A. Senna", 1000));
            listBindingDefault.Add(
                new RaceCarDriverDefault("A. Prost", 400));
        }
        #endregion

        #region Default Constructor for Item Class

        BindingManagerBase BindingManagerDefault
        {
            get { return this.BindingContext[this.listBindingDefault]; }
        }

        RaceCarDriverDefault CurrentDefault
        {
            get { return (RaceCarDriverDefault)BindingManagerDefault.Current; }
        }
        private void InitializeGridBindingDefault()
        {
            InitializeBindingDefault();

            dataGridViewDefault.DataSource = listBindingDefault;

            nameDefaultTextBox.DataBindings.Add("Text", listBindingDefault, "Name");
            winsDefaultTextBox.DataBindings.Add("Text", listBindingDefault, "Wins");
        }

        private void buttonAddDefault_Click(object sender, EventArgs e)
        {
            try
            {
                listBindingDefault.AddNew();
            }
            catch (MissingMethodException mme)
            {
                MessageBox.Show("Error: no default constructor\n\r" + mme);
            }
            catch (InvalidOperationException ioe)
            {
                MessageBox.Show("Error: no default constructor\n\r" + ioe);
            }
        }

        private void buttonDefault_Click(object sender, EventArgs e)
        {
            listBindingDefault.Remove(CurrentDefault);
        }

        private void buttonAddWinsDefault_Click(object sender, EventArgs e)
        {
            CurrentDefault.Wins++;
        }

        private void buttonFirstDefault_Click(object sender, EventArgs e)
        {
            BindingManagerDefault.Position = 0;
        }

        private void buttonPreviousDefault_Click(object sender, EventArgs e)
        {
            BindingManagerDefault.Position--;
        }

        private void buttonNextDefault_Click(object sender, EventArgs e)
        {
            BindingManagerDefault.Position++;
        }

        private void buttonLastDefault_Click(object sender, EventArgs e)
        {
            BindingManagerDefault.Position =
                BindingManagerDefault.Count - 1;
        }
        #endregion

    }
}